/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.twa.filter.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * HTML ブロック・マッチャーです。
 */
public class HtmlBlockMatcher {

	private static final String HTML4_STRICT_BLOCK_TAGS = "p|div|table|dl|ul|ol|form|address|blockquote|h[1-6]|fieldset|hr|pre"; //$NON-NLS-1$
	private static final String ADDITIONAL_BLOCK_TAGS = "body|head|meta|link|script|li|title|tr|th|td|thead|tbody|dt|dd"; //$NON-NLS-1$
	private static final String ALL_BLOCK_TAGS = "(" + ADDITIONAL_BLOCK_TAGS + "|" + HTML4_STRICT_BLOCK_TAGS + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	private static final String BLOCK_TAG_PATTERN = "(?i)(?s)<(|/)" + ALL_BLOCK_TAGS + "(>|\\s+[^>]*>)"; //$NON-NLS-1$ //$NON-NLS-2$

	private static final Pattern blockTagPattern = Pattern.compile(BLOCK_TAG_PATTERN + "(.+?)" + BLOCK_TAG_PATTERN); //$NON-NLS-1$

	private final Matcher matcher;

	public HtmlBlockMatcher(String input) {
		matcher = blockTagPattern.matcher(input);
	}

	public boolean find(int start) {
		return matcher.find(start);
	}

	public int start() {
		return matcher.start();
	}

	public String group() {
		return matcher.group();
	}

	public static String trimTag(String s, String tagPattern) {
		s = s.replaceFirst("(?i)(?s)^" + tagPattern + "\\s*", ""); // 先頭タグ除去 //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		s = s.replaceFirst("(?i)(?s)\\s*" + tagPattern + "$", ""); // 後方タグ除去 //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		return s;
	}

	public static String trimBlockTag(String s) {
		s = trimTag(s, BLOCK_TAG_PATTERN);
		s = trimTag(s, "<br\\s*(|/)\\s*>"); //$NON-NLS-1$
		s = trimTag(s, "(&nbsp;)+"); //$NON-NLS-1$
		s = s.trim();
		return s;
	}
}
